/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.secure;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryException;
import org.wso2.registry.jdbc.DatabaseConstants;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.readwrite.DefaultDatabaseUtil;

import java.sql.*;

public class HSQLDBInitializer {

    private static final Log log = LogFactory.getLog(RegistryRealm.class);

    public void createHSQLTables() throws RegistryException {

        try {
            Class.forName(DatabaseConstants.HSQL_DRIVER_NAME);
        } catch (ClassNotFoundException e) {
            String msg = "Could not load the HSQL driver.";
            log.fatal(msg, e);
            throw new RegistryException(msg);
        }

        Connection conn = null;
        try {
            conn = DriverManager.getConnection(UserManagerConstants.HSQL_DB_URL, "sa", "");

        } catch (SQLException e) {
            String msg = "Could not get the HSQL DB connection for creating tables.";
            log.fatal(msg, e);
            throw new RegistryException(msg);
        }

        try {
            DatabaseMetaData metaData = conn.getMetaData();
            ResultSet result = metaData.getTables(null, null, "um_users", null);
            if(result.next()) {
                log.info("User Manager tables were already created.");
                return;
            }
            
        } catch (SQLException e) {
            String msg = "Could not get the metadata of the User Manager database.";
            log.fatal(msg, e);
            throw new RegistryException(msg);
        }

        try {
            
           DefaultDatabaseUtil.createDatabase(conn);
       
        }catch (UserManagerException e) {
            String msg = "Could not create tables for user manager";
            log.fatal(msg, e);
            throw new RegistryException(msg);
        }catch (SQLException e) {
            String msg = "Could not create tables for the User Manager.";
            log.fatal(msg, e);
            throw new RegistryException(msg);

        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignore) {}
            }
        }
    }
}
